package com.ruoyi.service.impl;

import java.util.List;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.framework.websocket.WebSocketUsers;
import com.ruoyi.service.UserPhotoStorageService;
import com.ruoyi.web.request.UserPhotoStorageCreateRequest;
import com.ruoyi.web.request.UserPhotoStorageListRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.mapper.UserPhotoStorageMapper;
import com.ruoyi.domain.UserPhotoStorage;
import org.springframework.transaction.annotation.Transactional;

/**
 * 用户照片存储Service业务层处理
 *
 * @author gxk
 * @date 2024-01-29
 */
@Service
@Transactional
public class UserPhotoStorageServiceImpl extends ServiceImpl<UserPhotoStorageMapper, UserPhotoStorage> implements UserPhotoStorageService
{
    @Autowired
    private UserPhotoStorageMapper userPhotoStorageMapper;

    /**
     * 查询用户照片存储列表
     *
     * @param request 用户照片存储
     * @return 用户照片存储
     */
    @Override
    public List<UserPhotoStorage> selectUserPhotoStorageList(UserPhotoStorageListRequest request)
    {
        return userPhotoStorageMapper.selectUserPhotoStorageList(request);
    }

    /**
     * 新增用户照片存储
     *
     * @param request 用户照片存储
     * @return 结果
     */
    @Override
    public int insertUserPhotoStorage(UserPhotoStorageCreateRequest request)
    {
        LoginUser loginUser = SecurityUtils.getLoginUser();
        UserPhotoStorage userPhotoStorage = UserPhotoStorage.builder()
                .userId(loginUser.getUserId())
                .username(loginUser.getUsername())
                .path(request.getPath())
                .createTime(DateUtils.getNowDate())
                .build();
        WebSocketUsers.sendMessageToUserByUserId(loginUser.getUserId(), request.getPath());
        return userPhotoStorageMapper.insert(userPhotoStorage);
    }

    /**
     * 修改用户照片存储
     *
     * @param userPhotoStorage 用户照片存储
     * @return 结果
     */
    @Override
    public int updateUserPhotoStorage(UserPhotoStorage userPhotoStorage)
    {
        return userPhotoStorageMapper.updateUserPhotoStorage(userPhotoStorage);
    }

}
